@@ -14,7 +14,7 @@
 //!
 //! This crate aims to replicate the convention of the __curl__ library and offer it
 //! behind a simple API: in most cases, a single function, `for_url()`, which accepts
-//! a target URL and returns the proxy parameters, if applicable. The method for determining 
+//! a target URL and returns the proxy parameters, if applicable. The method for determining
 //! the parameters is described in detail in that function's documentation.
 //!
 //! # Getting Started
@@ -22,10 +22,10 @@
 //! Add the following to the `[dependencies]` section of your `Cargo.toml`:
 //!
 //! ```toml
-//! env_proxy = "0.2"
+//! env_proxy = "0.3"
 //! ```
 //!
-//! Also, import the crate to your crate root:
+//! If you're using the 2015 edition of Rust, import the crate to your crate root:
 //!
 //! ```
 //! extern crate env_proxy;
@@ -52,9 +52,8 @@
 
 
 #[cfg(test)]
-#[macro_use] extern crate lazy_static;
-#[macro_use] extern crate log;
-extern crate url;
+use lazy_static::lazy_static;
+use log::warn;
 
 use std::env::var_os;
 use url::Url;
@@ -93,7 +92,7 @@ fn matches_no_proxy(url: &Url) -> bool {
                     if let Some(host_ch) = host_iter.next() {
                         let host_ch = host_ch as u32;
                         let elem_ch = match elem_ch as u32 {
-                            uppercase @ 0x41 ... 0x5a => uppercase + 0x20,
+                            uppercase @ 0x41 ..= 0x5a => uppercase + 0x20,
                             anything => anything
                         };
                         if elem_ch == host_ch {
@@ -155,7 +154,8 @@ impl ProxyUrl {
     ///
     /// The transformation will:
     ///
-    /// * Parse the raw URL as a `Url`;
+    /// * Parse the raw URL as a `Url`. If the raw URL lacks the scheme, `http` is assumed and
+    ///   "http://" is prepended to canonicalize the value;
     /// * Ensure that the host part is not empty;
     /// * Use the default value for the port (or not, see [`with_default_port()`](#method.with_default_port))
     ///   if one is not specified in the raw URL.
@@ -163,14 +163,38 @@ impl ProxyUrl {
     ///
     /// If any of the steps fail, `None` will be returned.
     pub fn to_url(self) -> Option<Url> {
-        if let Some(Ok(mut url)) = self.0.map(|s| Url::parse(&s).map_err(|e| {
-            warn!("url parse error: {}", e);
-            e
-        })) {
+        let mut orig_scheme = self.0.as_ref().map(|s|
+            if s.starts_with("http://") {
+                Some("http")
+            } else if s.starts_with("https://") {
+                Some("https")
+            } else {
+                None
+            }
+        ).unwrap_or(None);
+        if let Some(Ok(mut url)) = self.0.map(|mut s| {
+            if !s.contains("://") {
+                s.insert_str(0, "http://");
+                orig_scheme = Some("http");
+            }
+            if orig_scheme.is_some() {
+                s = s.replacen("http", "xttp", 1);
+            }
+            Url::parse(&s).map_err(|e| {
+                warn!("url parse error: {}", e);
+                e
+            })
+        }) {
             if url.host_str().is_none() {
                 warn!("host part of the URL is empty");
                 return None;
             }
+            if let Some(orig_scheme) = orig_scheme {
+                if url.set_scheme(orig_scheme).is_err() {
+                    warn!("could not set URL scheme back to {}", orig_scheme);
+                    return None;
+                }
+            }
             if url.port().is_some() {
                 return Some(url);
             } else if self.1.is_none() {
@@ -233,17 +257,18 @@ impl ProxyUrl {
 /// |xample.org   |  &#x2718; |  &#x2718;  |
 /// |org          |  &#x2718; |  &#x2718;  |
 ///
-/// For the __ftp__ protocol scheme, __ftp_proxy__ is checked first; for __https__, __https_proxy__
-/// is checked. Both schemes will fall back to __http_proxy__, then __all_proxy__ if the former is
-/// undefined. For __http__, __http_proxy__ is cheked first, then __all_proxy__. For all other schemes
-/// only __all_proxy__ is checked. In this context, "checked" means that the value of a variable is used
-/// if present, and the search for further definitions stops.
+/// For the __ftp__ protocol scheme, __ftp_proxy__ is checked first; for __http__, __http_proxy__
+/// is checked, and for __https__, it's __https_proxy__. These three schemes will fall back to __all_proxy__
+/// if the original variable is undefined. For all other schemes only __all_proxy__ is checked. In this
+/// context, "checked" means that the value of a variable is used if present, and the search for further
+/// definitions stops.
 ///
-/// The return value, if not `None`, is an opaque structure wrapping the raw value of the chosen
-/// environment variable.
+/// The return value, if not `None`, is an opaque structure wrapping the value (possibly canonicalized,
+/// see [`ProxyUrl::to_url()`](struct.ProxyUrl.html#method.to_url)) of the chosen environment variable.
 ///
 /// If the target URL matches __no_proxy__, or if the hostname cannot be extracted from the URL,
 /// the function returns `None`. If the port is not explicitly defined in the proxy URL, the value 8080
+/// is used.
 pub fn for_url(url: &Url) -> ProxyUrl {
     if matches_no_proxy(url) {
         return ProxyUrl(None, None);
@@ -255,9 +280,9 @@ pub fn for_url(url: &Url) -> ProxyUrl {
     let maybe_all_proxy = env_var_pair!("all_proxy", "ALL_PROXY");
 
     let url_value = match url.scheme() {
-        "https" => maybe_https_proxy.or(maybe_http_proxy.or(maybe_all_proxy)),
+        "https" => maybe_https_proxy.or(maybe_all_proxy),
         "http" => maybe_http_proxy.or(maybe_all_proxy),
-        "ftp" => maybe_ftp_proxy.or(maybe_http_proxy.or(maybe_all_proxy)),
+        "ftp" => maybe_ftp_proxy.or(maybe_all_proxy),
         _ => maybe_all_proxy,
     };
     ProxyUrl(url_value, Some(8080))
@@ -367,7 +392,39 @@ mod tests {
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
         assert_eq!(
             for_url_str("http://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8080/".to_string()))
+            Some("http://proxy.example.com:8080/".to_string())
+        );
+    }
+
+    #[test]
+    fn default_proxy_url_scheme() {
+        let _l = LOCK.lock();
+        scrub_env();
+        set_var("http_proxy", "proxy.example.com:8080");
+        let u = Url::parse("http://www.example.org").ok().unwrap();
+        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
+        assert_eq!(
+            for_url_str("http://www.example.org").to_string(),
+            Some("http://proxy.example.com:8080/".to_string())
+        );
+    }
+
+    #[test]
+    fn proxy_url_with_explicit_scheme_port() {
+        let _l = LOCK.lock();
+        scrub_env();
+        set_var("http_proxy", "http://proxy.example.com:80");
+        let u = Url::parse("http://www.example.org").ok().unwrap();
+        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 80)));
+        assert_eq!(
+            for_url_str("http://www.example.org").to_string(),
+            Some("http://proxy.example.com:80/".to_string())
+        );
+        set_var("http_proxy", "https://proxy.example.com:443");
+        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 443)));
+        assert_eq!(
+            for_url_str("http://www.example.org").to_string(),
+            Some("https://proxy.example.com:443/".to_string())
         );
     }
 
@@ -380,13 +437,13 @@ mod tests {
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
         assert_eq!(
             for_url_str("http://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8080/".to_string()))
+            Some("http://proxy.example.com:8080/".to_string())
         );
         set_var("all_proxy", "http://proxy.example.org:8081");
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.org".to_string(), 8081)));
         assert_eq!(
             for_url_str("http://www.example.org").to_string(),
-            Some(("http://proxy.example.org:8081/".to_string()))
+            Some("http://proxy.example.org:8081/".to_string())
         );
     }
 
@@ -395,19 +452,18 @@ mod tests {
         let _l = LOCK.lock();
         scrub_env();
         set_var("HTTPS_PROXY", "http://proxy.example.com:8080");
-        set_var("http_proxy", "http://proxy.example.org:8081");
         set_var("all_proxy", "http://proxy.example.org:8081");
         let u = Url::parse("https://www.example.org").ok().unwrap();
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
         assert_eq!(
             for_url_str("https://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8080/".to_string()))
+            Some("http://proxy.example.com:8080/".to_string())
         );
-        set_var("https_proxy", "http://proxy.example.com:8081");
-        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8081)));
+        set_var("https_proxy", "http://proxy.example.com:8082");
+        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8082)));
         assert_eq!(
             for_url_str("https://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8081/".to_string()))
+            Some("http://proxy.example.com:8082/".to_string())
         );
     }
 
@@ -415,25 +471,18 @@ mod tests {
     fn https_proxy_fallback() {
         let _l = LOCK.lock();
         scrub_env();
-        set_var("http_proxy", "http://proxy.example.com:8080");
         set_var("ALL_PROXY", "http://proxy.example.org:8081");
-        let u = Url::parse("https://www.example.org").ok().unwrap();
-        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
-        assert_eq!(
-            for_url_str("https://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8080/".to_string()))
-        );
-        remove_var("http_proxy");
+        let u = Url::parse("ftp://www.example.org").ok().unwrap();
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.org".to_string(), 8081)));
         assert_eq!(
             for_url_str("https://www.example.org").to_string(),
-            Some(("http://proxy.example.org:8081/".to_string()))
+            Some("http://proxy.example.org:8081/".to_string())
         );
         set_var("all_proxy", "http://proxy.example.org:8082");
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.org".to_string(), 8082)));
         assert_eq!(
             for_url_str("https://www.example.org").to_string(),
-            Some(("http://proxy.example.org:8082/".to_string()))
+            Some("http://proxy.example.org:8082/".to_string())
         );
     }
 
@@ -442,19 +491,18 @@ mod tests {
         let _l = LOCK.lock();
         scrub_env();
         set_var("FTP_PROXY", "http://proxy.example.com:8080");
-        set_var("http_proxy", "http://proxy.example.org:8081");
         set_var("all_proxy", "http://proxy.example.org:8081");
         let u = Url::parse("ftp://www.example.org").ok().unwrap();
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
         assert_eq!(
             for_url_str("ftp://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8080/".to_string()))
+            Some("http://proxy.example.com:8080/".to_string())
         );
-        set_var("ftp_proxy", "http://proxy.example.com:8081");
-        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8081)));
+        set_var("ftp_proxy", "http://proxy.example.com:8082");
+        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8082)));
         assert_eq!(
             for_url_str("ftp://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8081/".to_string()))
+            Some("http://proxy.example.com:8082/".to_string())
         );
     }
 
@@ -462,25 +510,18 @@ mod tests {
     fn ftp_proxy_fallback() {
         let _l = LOCK.lock();
         scrub_env();
-        set_var("http_proxy", "http://proxy.example.com:8080");
         set_var("ALL_PROXY", "http://proxy.example.org:8081");
         let u = Url::parse("ftp://www.example.org").ok().unwrap();
-        assert_eq!(for_url(&u).host_port(), Some(("proxy.example.com".to_string(), 8080)));
-        assert_eq!(
-            for_url_str("ftp://www.example.org").to_string(),
-            Some(("http://proxy.example.com:8080/".to_string()))
-        );
-        remove_var("http_proxy");
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.org".to_string(), 8081)));
         assert_eq!(
             for_url_str("ftp://www.example.org").to_string(),
-            Some(("http://proxy.example.org:8081/".to_string()))
+            Some("http://proxy.example.org:8081/".to_string())
         );
         set_var("all_proxy", "http://proxy.example.org:8082");
         assert_eq!(for_url(&u).host_port(), Some(("proxy.example.org".to_string(), 8082)));
         assert_eq!(
             for_url_str("ftp://www.example.org").to_string(),
-            Some(("http://proxy.example.org:8082/".to_string()))
+            Some("http://proxy.example.org:8082/".to_string())
         );
     }
 }
